/*24 Bit Sun.c
  Reader for 24 bit sun raster files.
  September 28, 1989
  Eric Pepke
*/

#include <MacTypes.h>
#include <FileMgr.h>
#include <MemoryMgr.h>

/*Raster file header*/
typedef struct
	{
		long magicNumber;	/*Magic number, must be 0x59a66a95*/
		long width;			/*Width of the image in pixels*/
		long height;		/*Height of the image in pixels*/
		long depth;			/*Number of bits in pixel, must be 24*/
		long length;		/*Number of bytes in each plane--not used*/
		long type;			/*Type of file, must be 0 or 1*/
		long mapType;		/*Type of color map, may be 0 or 2*/
		long mapLength;		/*Length of color map, must be 0*/
	} RasHeader;

/*Additional errors*/
#define RGBBadSize		29500	/*Size is different from what was specified*/
#define RGBBadFormat	29501	/*Format is bad or unrecognized*/

pascal OSErr main(fileName, vRefNum, width, height, data, params)
StringPtr fileName;
short vRefNum;
short *width, *height;
Handle data, params;
/*Reads in a 24-bit Sun raster file according to section 7.2 of the RGBView
  guide.  This version does not use params*/
{
	ParamBlockRec theParams;	/*The io params for low-level file calls*/
	OSErr theError;				/*The last error returned by a toolbox routine*/
	RasHeader theFileHeader;	/*The header of the file*/
	long expectedLength;		/*The expected length of the file*/
	short testWidth;			/*Width of the image for testing the file length*/
	short whichLine;			/*Which line we're reading*/
	long dataOffset;			/*The offset into the data*/
	
	/*Get ready to open the file*/
	theParams . ioParam . ioCompletion = 0;
	theParams . ioParam . ioNamePtr = fileName;
	theParams . ioParam . ioVRefNum = vRefNum;
	theParams . ioParam . ioVersNum = 0;
	theParams . ioParam . ioPermssn = fsRdPerm;
	theParams . ioParam . ioMisc = 0;
	
	/*Open the file*/
	theError = PBOpen(&theParams, false);
	if (theError)
	{
		return theError;
	}
	
	/*Read the header*/
	theParams . ioParam . ioBuffer = (Ptr) &theFileHeader;
	theParams . ioParam . ioReqCount = sizeof(RasHeader);
	theParams . ioParam . ioPosMode = fsFromStart;
	theParams . ioParam . ioPosOffset = 0;
	theError = PBRead(&theParams, false);
	if (theError)
	{
		PBClose(&theParams, false);
		return theError;
	}
	
	/*Check the header for sanity*/
	if (theFileHeader . magicNumber != 0x59a66a95 ||
		theFileHeader . depth != 24 ||
		(theFileHeader . type != 0 && theFileHeader . type != 1) ||
		(theFileHeader . mapType != 0 && theFileHeader . mapType != 2) ||
		theFileHeader . mapLength != 0)
	{
		PBClose(&theParams, false);
		return RGBBadFormat;
	}
	
	/*Check the length of the file for sanity*/
	testWidth = theFileHeader . width;
	if (testWidth & 1)
	{
		++testWidth;
	}
	expectedLength = theFileHeader . height * testWidth * 3L + sizeof(RasHeader);
	theError = PBGetEOF(&theParams, false);
	if (theError)
	{
		PBClose(&theParams, false);
		return theError;
	}
	if (((long) theParams . ioParam . ioMisc) != expectedLength)
	{
		PBClose(&theParams, false);
		return RGBBadFormat;
	}

	/*Check the width and height*/
	if (*width < 0)
	{
		*width = theFileHeader . width;
	}
	else if (*width != theFileHeader . width)
	{
		return RGBBadSize;
	}
	if (*height < 0)
	{
		*height = theFileHeader . height;
	}
	else if (*height != theFileHeader . height)
	{
		return RGBBadSize;
	}

	/*If data is a handle to something, read the data*/
	if (data)
	{
		/*Make the data big enough*/
		SetHandleSize(data, theFileHeader . width * theFileHeader . height * 3L);
		if (theError = MemError())
		{
			PBClose(&theParams, false);
			return theError;
		}

		/*Read every line*/
		dataOffset = 0;
		for (whichLine = 0; whichLine < theFileHeader . height; ++whichLine)
		{
			/*Read in a line*/
			HLock(data);
			theParams . ioParam . ioBuffer = *data + dataOffset;
			theParams . ioParam . ioReqCount = theFileHeader . width * 3L;
			theParams . ioParam . ioPosMode = fsAtMark;
			theError = PBRead(&theParams, false);
			HUnlock(data);
			if (theError)
			{
				PBClose(&theParams, false);
				return theError;
			}

			/*Move the offset to the next line*/
			dataOffset += theFileHeader . width * 3L;

			/*If the width is odd, skip 3 bytes in the file*/
			if (theFileHeader . width & 1)
			{
				theParams . ioParam . ioPosMode = fsFromMark;
				theParams . ioParam . ioPosOffset = 3;
				theError = PBSetFPos(&theParams, false);
				if (theError)
				{
					PBClose(&theParams, false);
					return theError;
				}
			}
		}
	}

	return PBClose(&theParams, false);
}